import os
from datetime import datetime
from pathlib import Path

from .templates import default


def today() -> Path:
    today = datetime.today().strftime("%Y-%m-%d")
    today_path = Path().home() / ".plan" / f"{today}.md"
    return today_path


def print_plan(plan: Path):
    import subprocess

    cat_process = subprocess.Popen(["cat", str(plan)], stdout=subprocess.PIPE)
    pandoc_process = subprocess.Popen(
        ["pandoc", "-t", "pdf", "-V", "geometry:margin=.5in", "-"],
        stdin=cat_process.stdout,
        stdout=subprocess.PIPE,
    )
    subprocess.run(
        ["lp", "-o", "sides=two-sided-long-edge"],
        # , "-o", "number-up=4"],
        stdin=pandoc_process.stdout,
    )


def template_or_open(
    plan: Path, name: str = "plan.html", should_open: bool = True
) -> str | None:
    template = default(name)

    if should_open:
        import subprocess

        plan.parent.mkdir(parents=True, exist_ok=True)
        if not plan.exists():
            plan.touch()
        with plan.open("w") as f:
            f.write(template)

        editor = os.getenv("EDITOR", "nvim")
        os.chdir(plan.parent)
        subprocess.run([editor, str(plan)])
    else:
        return template


def open_in_editor(plan: Path):
    import subprocess

    plan.parent.mkdir(parents=True, exist_ok=True)
    if not plan.exists():
        plan.touch()
        with plan.open("w") as f:
            f.write(default())
    editor = os.getenv("EDITOR", "nvim")
    os.chdir(Path().home() / ".plan")
    subprocess.run([editor, str(plan)])
