from pathlib import Path
from pydantic import BaseModel, ConfigDict
from typing import Iterable, Optional
import click
from . import open_in_editor, template_or_open, today, print_plan
import markdown
from io import StringIO
from click_default_group import DefaultGroup


@click.group(cls=DefaultGroup, default="today", default_if_no_args=True)
def cli():
    pass


@cli.command("print")
def print_today():
    path = today()
    print_plan(path)


@cli.command("todo")
def todo():
    import sys
    from lxml import etree

    class Result(BaseModel):
        path: Path
        todos: Iterable[etree.Element]
        model_config = ConfigDict(arbitrary_types_allowed=True)

    def parse_todo() -> Iterable[Result]:
        for m in recent():
            with open(m) as f:
                text = f.read()
                content = markdown.markdown(text)
                tree = etree.parse(
                    StringIO(f"<root>{content}</root>"), etree.HTMLParser()
                )
                for node in tree.xpath("//h2/following-sibling::ul/li"):
                    yield Result(path=m, todos=node)

    todos = parse_todo()

    def print_tree(node: Optional[etree.Element], tabs=0):
        if node is None:
            return
        print(f"{' ' * tabs}- {node.text}")
        for child in node.iterchildren():
            return print_tree(child, tabs + 1)

    for day in todos:
        for root in day.todos:
            print(f"{day.path.name}")
            print_tree(root)
    else:
        print("no todos")

    sys.exit(0)
    """

    uv run plan todo

    interact
    import wat
    print(node)

    r = next(root)
    r

    wat(node)
    dir(node)
    wat(node)
    [s for s in node.itersiblings()]
    [s for s in node.iterchildren()]
    print(node.tag)
    parent = node.xpath("//following-sibling::ul")[0]
    [s.text for s in parent.iterchildren()]

    sys.exit(0)

    """


@cli.command("today")
@click.option(
    "-t",
    "--template",
    "template",
    required=True,
    type=click.Choice(["plan.html", "work.html", "blank.html", "print.html"]),
    prompt=True,
    default="plan.html",
    show_default=True,
)
@click.option(
    "--print/--no-print",
    "should_print",
    is_flag=True,
    default=False,
    prompt="print template only?",
    required=True,
)
def open_today(template, should_print):
    """open today"""
    path = today()
    if should_print:
        print(template_or_open(path, should_open=False, name=template))
    else:
        open_in_editor(path)


def recent() -> Iterable[Path]:
    parent = Path().home() / ".plan"
    for m in parent.iterdir():
        if m.suffix == ".md":
            yield m
        else:
            yield m
